// ClientSocket.cpp : implementation file
//

#include "stdafx.h"
#include "CodeWarHost_1.h"
#include "ClientSocket.h"
#include "ServerSocket.h"
#include <iomanip>
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CClientSocket

CClientSocket::CClientSocket(basic_arena &a, CServerSocket *p):arena(a), parent(p)
{
}

CClientSocket::~CClientSocket()
{
}


// Do not edit the following lines, which are needed by ClassWizard.
#if 0
BEGIN_MESSAGE_MAP(CClientSocket, CSocket)
	//{{AFX_MSG_MAP(CClientSocket)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#endif	// 0

/////////////////////////////////////////////////////////////////////////////
// CClientSocket member functions

void CClientSocket::OnConnect(int nErrorCode) 
{
	CSocket::OnConnect(nErrorCode);
}

void CClientSocket::OnReceive(int nErrorCode) 
{
    using namespace codewar;
    CString msg;
    client_handshake_message m;
    msg_head head;
    Receive(&head, sizeof(head), MSG_PEEK);
    switch(head.id){
    case msgid_client_handshake:
        Receive(&m, sizeof(m));
        c_handshake = m.handshake;
        msg.Format("Received client %d handshake, species name = [%s]", c_handshake.species_id, c_handshake.species_name);
        DisplayLine(msg);
        break;
    case msgid_creature_stats:
        ReceiveCreatureStats();
        break;
    case msgid_creature_action:
        ReceiveCreatureAction();
        break;
    default:
        Flush();
        break;
    }
	CSocket::OnReceive(nErrorCode);
}

void CClientSocket::OnClose(int nErrorCode) 
{
    CString msg;
    msg.Format("Connection to %d lost", s_handshake.species_id);
    DisplayLine(msg);
	CSocket::OnClose(nErrorCode);
}

bool CClientSocket::ReceiveCreatureStats()
{
    using namespace codewar;
    creature_stats_message msg;
    Receive(&msg, sizeof(msg));
    stats = msg.stats;
    CString s;
    s.Format("Received stats %d, %d, %d, %d from %s", stats.attack, stats.defense, stats.food_consumption, stats.max_life, c_handshake.species_name);
    DisplayLine(s);
    return true;
}

void CClientSocket::Flush()
{
    using namespace std;
    char c;
    CString s;
    s.Format("Flushing socket for client %d", s_handshake.species_id);
    DisplayLine(s);
    cout<<"Flushing socket for client "<<s_handshake.species_id<<", content was:"<<endl;
    int i = 0;
    while(Receive(&c, 1)){
        cout<<setw(2)<<setfill('0')<<hex<<(int)c<<" ";
        if(++i == 20){
            cout<<endl;
            i = 0;
        }
    }
    cout<<endl;
}

bool CClientSocket::SendCreatureFeedback(int i)
{
    using namespace codewar;
    basic_vision_info vision_info;
    ComposeVisionInfo(vision_info, i);
    creature_feedback_message message(creatures[i].data, 
        creatures[i].attacked, creatures[i].last_turn_result, 
        vision_info);
    std::cout<<"Sending creature feedback for species "<<s_handshake.species_id<<", creature "<<i<<std::endl;
    creatures[i].next_turn_action = idle;
    return (message.size() == Send(message.message(), message.size()));
}


void CClientSocket::ComposeVisionInfo(codewar::basic_vision_info &v, int i)
{
    for(int cell_index = 0; cell_index<8; ++cell_index){
        basic_cell cell = arena.get_relative_cell(creatures[i].data.x, 
                        creatures[i].data.y, 
                        creatures[i].data.facing, 
                        cell_index);
        if(cell.contents != codewar::cell_creature){
            v.push_back(cell.contents);
        }
        else{
            v.push_back(codewar::cell_creature);
            codewar::sighted_creature_info creature_info; 
            parent->GetSightedCreatureInfo(creature_info, cell.species, cell.creature);
            v.push_back(creature_info);
        }
    }
}

bool CClientSocket::ReceiveCreatureAction()
{
    using namespace codewar;
    creature_action_message action;
    VERIFY(sizeof(action) == Receive(&action, sizeof(action)));
    creatures[action.creature].next_turn_action = action.action;
    parent->MakeTurn();
    return true;
}



void CClientSocket::SendFeedback()
{
    for(int i=0; i<creatures.size(); ++i){
        VERIFY(SendCreatureFeedback(i));
    }
}
